/*
 * vi:ts=4:shiftwidth=4:expandtab
 * vim600:fdm=marker
 * SWIG binding for python
 * Last Change : 01-Nov-2004.
 */
%module cmaxent

%{
#include "maxentmodel.hpp"
%}
%include "std_string.i"

namespace maxent {
    using namespace std;

    extern int verbose;  // set this to 0 if you do not want verbose output

    class MaxentModel {
        public:
            // remove them?
            // typedef me::feature_type feature_type;
            // typedef me::feature_type outcome_type;
            typedef std::string outcome_type;
            typedef std::string feature_type;
            typedef std::vector<pair<feature_type, float> > context_type;

            %typemap (in) context_type& (context_type v) { // {{{
                int len;
                int i;

                if (!PyList_Check($input)) 
                    PyErr_Print();

                len  = PyList_Size($input);
                if (len > 0) {
                    PyObject* x = PyList_GetItem($input, 0);

                    // if this is a tuple of (string, float) {{{
                    if (PyTuple_Check(x) && PyTuple_Size(x) == 2) {
                        PyObject* a;
                        PyObject* b;
                        char* s;
                        double d;
                        for (i = 0; i < len; ++i) {
                            x = PyList_GetItem($input, i);
                            a = PyTuple_GetItem(x, 0);
                            b = PyTuple_GetItem(x, 1); // TODO: use PyTuple_GET_ITEM() for speed
                            assert(a); 
                            assert(b);
                            s = PyString_AsString(a);
                            if (PyErr_Occurred())
                                PyErr_Print();
                            d = PyFloat_AsDouble(b);
                            if (PyErr_Occurred())
                                PyErr_Print();
                            v.push_back(std::make_pair(std::string(s), d));
                        }
                        // }}}

                        // or just a single string, float value is omitted {{{
                } else if (PyString_Check(x)) {
                    char* s;
                    for (i = 0; i < len; ++i) {
                        x = PyList_GetItem($input, i);
                        s = PyString_AsString(x);
                        if (PyErr_Occurred())
                            PyErr_Print();
                        v.push_back(std::make_pair(std::string(s), 1.0));
                    }
                    // }}}

                } else {
                    PyErr_SetString(PyExc_RuntimeError, "invalid context_type argument, should be [(str, float), ...]");
                }
                }

                $1 = &v;
            }// }}}

            %typemap (out) std::vector<pair<outcome_type, double> > { // {{{
                PyObject* tup;
                PyObject* ret = PyList_New($1.size());
                assert(ret);
                for (size_t i = 0; i < $1.size(); ++i) {
                    tup = PyTuple_New(2);
                    assert(tup);
                    if (!tup) {
                        PyErr_SetString(PyExc_RuntimeError, "out of memory in calling PyTuple_New().");
                    }
                    PyTuple_SetItem(tup, 0,
                            PyString_FromString($1[i].first.c_str()));
                    PyTuple_SetItem(tup, 1, PyFloat_FromDouble($1[i].second));
                    PyList_SetItem(ret, i, tup);
                    if (PyErr_Occurred())
                        PyErr_Print();
                }
                $result = ret;
            } // }}}

            MaxentModel();

            char* __str__() const;

            void load(const string& model);

            void save(const string& model, bool binary = false) const;

            double eval(const context_type& context, const outcome_type& outcome) const;

            outcome_type predict(const context_type& context) const;

            void begin_add_event();

            void add_event(const context_type& context,
                    const outcome_type& outcome,
                    size_t count = 1);

            void add_heldout_event(const context_type& context,
                    const outcome_type& outcome,
                    size_t count = 1);

            void end_add_event(size_t cutoff = 1);

            void train(size_t iter = 15, const std::string& method = "lbfgs",
                    double sigma = 0.0, // non-zero enables Gaussian prior smoothing
                    double tol = 1E-05);

            %name(eval_all) std::vector<pair<outcome_type, double> > py_eval(const context_type& context) const;
    };


} // namespace maxent

